/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef FTIMERS_H
#define FTIMERS_H

#include "fpm.h"
#include "wObject.h"
#include <map>
#include <chrono>

namespace fpm
{

/**
 * @brief Basic and portable timer
 * @author Adrien Crovato
 * @todo use clock_gettime for POSIX (https://en.cppreference.com/w/cpp/chrono/c/clock)?
 */
class FPM_API Timer
{
private:
    bool isOn;                                         ///< status
    std::chrono::high_resolution_clock::time_point tS; ///< start time (wall-clock)
    std::chrono::high_resolution_clock::time_point tE; ///< stop time (wall-clock)
    std::clock_t cS;                                   ///< start time (cpu)
    std::clock_t cE;                                   ///< stop time (cpu)

public:
    Timer();
    virtual ~Timer() {}

    void reset();
    void start();
    void stop();
    inline double getWall() const;
    inline double getCpu() const;
};

#include "fTimers.inl"

/**
 * @brief Collection of timers
 * @author Romain Boman, Adrien Crovato
 */
class FPM_API Timers : public fwk::wObject
{
private:
    std::map<std::string, Timer> timers; ///< set of timers

public:
    Timers() : fwk::wObject() {}
    virtual ~Timers() {}

#ifndef SWIG
    Timer &operator[](std::string const &name);
    virtual void write(std::ostream &out) const override;
#endif //SWIG
};

} // namespace fpm

#endif //FTIMERS_H
